@@ -74,7 +74,7 @@ def logger(is_timed: bool):
     return decorator
 
 
-BASE_URL = environ.get("CHATGPT_BASE_URL") or "https://gpt.pawan.krd/backend-api/"
+BASE_URL = environ.get("CHATGPT_BASE_URL") or "https://bypass.duti.tech/api/"
 
 
 class ErrorType:
@@ -580,7 +580,7 @@ class Chatbot:
 
                 raise Error(
                     source="ask",
-                    message="Field missing",
+                    message=line,
                     code=ErrorType.SERVER_ERROR,
                 )
             message = line["message"]["content"]["parts"][0]
